
= JavaScript Template of Amrita2

(caution: This is a experimental feature and not fixed)

Amrita2 supports JavaScripte Template.

Basically, it is a translator from HTML to JavaScript function running
on Ruby. And the API and result is very simmilar to Amrita2 Ruby
feature.

== Sample

This is a sample code.

      require 'amrita2/template'
      require 'amrita2/js'
      js = JS::CodeGenerator.new
      t = TemplateText.new '<x><em id="aaa">xxx</em></x>' 

      js.define_function(:sample) do 
        t.js_template('t', js)
        @js.code <<-END
        var tmpl = new t() ; 
        var e = document.createElement('div') ; 
        tmpl.expand(e, { 'aaa': '12' }) ;
        // e is  '<div><x><EM id=aaa>12</EM></x></div>'
        return e ;
        END
      end
      print js.results.join(NL)

The result is a JavaScript code.

  function sample() {
    function t() {
      // .. template code for <x><em id="aaa">xxx</em></x>
    }
    var e = document.createElement('div') ; 
    var tmpl = new t() ; 
    tmpl.expand(outer, { 'aaa': '12' }) ;
    // e is  '<div><x><EM id=aaa>12</EM></x></div>'
    return e ;
  }


   t.js_template('t', js)

makes a JavaScript function named 't'. It is very similar to Amrita2
Ruby Template Object.

    tmpl.expand(outer, { 'aaa': '12' }) ;

returns a DOM Element object which is expanded from template and
PO (Presentation Object).

In this case, it is 

  <div><x><EM id=aaa>12</EM></x></div>

== How to use it

* Generate a JavaScript source file statically.
* Generate a JavaScript source and embed it into static HTML file
* Generate a JavaScript source dynamically on demand
* Generate a JavaScript source dynamically on demand and embed it into also dynamically generated HTML page

In every case, details of UI is a HTML text. Your JavaScript code dose
not handle DOM functions or HTML tags. Only you should do is pass a PO
object like this 

  { 'aaa': '12' }

to the template object.

== Stauts and Roadmap

I'd like to make this feature full compatible with Amrita2 Ruby
template system. And I think I can make it because JavaScript has
enough feature for Amrita2 Template.

But I don't know JavaScript Language well now. So the current version
is very restricted and buggy.


== Hello World

--- include ../sample/js/hello.rb

== Ajax with Amrita2

--- include ../sample/js/json_ajax.rb

== Ajax with Amrita2 and Ruby on Rails

sample/rails/todoajax/ contains a Ajax version of ((<Rails Tutorial|URL:http://darkhost.mine.nu:81/~vince/rails/tutorial.html>))

Function and appearance is same but there is a important difference:
this version refreshes every 5 seconds the TODO list
asynchronously. If you typed some text for a new TODO, the text field
dose not lose foucus and the text you typed will be kept when
refreshing is done.

The view related code of this sample is here.

  def json_list_script
    tmpl = Amrita2::JS::TemplateText.new <<-END
    <div>
    Not done:
    <div id='not_done'>
       <input checked="checked" id="item_done" name="item[done]" type="checkbox" value="0" />
       <span> </span>
       <span id='description' />
       <span> </span>
       <a id='edit_link' href="/todo/edit/">Edit</a> 
       .....
    END
    # compiling the template to JavaScript functions
    js = Amrita2::JS::CodeGenerator.new
    tmpl.js_template('tmpl', js)
    Amrita2::JS::RUNTIME_SRC + js.results.join("\n") 
  end

  def json_list_po
    {
      :output=>true,
      :script=>Amrita2::SanitizedString[json_list_script]
    }
  end

Using this template, the static JavaScript code below display the page.

  function display_item(t, done, item) 
  {
    var onclick = 'document.location.href="/todo/toggle_check/' ;
    onclick += item.id  + '"' ;
    var edit_url = "/todo/edit/" + item.id ;
    var destroy_url = "/todo/destroy/" + item.id ;
    var destroy_confirm = 'return confirm("Are you sure you want to delete this entry: ' + item.description + '")' ;
    var f = function(t) {
       if (! ie_flag) {
         t.item_done('',{'onclick': onclick }) ;
       }
       t.description(item.description) ;
       t.edit_link('Edit', { 'href' : edit_url } ) ;
       t.destroy_link('Destroy', { 'href' : destroy_url, 'onclick': destroy_confirm } ) ;
    }
    if (done) {
      t.done(f) ;
    } else {
      t.not_done(f) ;
    }
  }
  
  
  // Use the function compiled from from the template
  function expand_template(data)
  { 
    // get the element to which the template will be inserted
    var e = document.getElementById('output') ;
    while (e.hasChildNodes()) {
        e.removeChild(e.firstChild) ;
    }
    // make a template object
    var t = new tmpl() ;
    // add changing data for demo
    var d = new Date()
    data.now = d.getMinutes() + ":" + d.getSeconds() ;
    // mix the data and the template
    t.expand(e, 
      function(t) {
        var not_done = data.not_done ;
        for (var i = 0 ; i < not_done.length ; i++) {
          var item = not_done[i] ;
          display_item(t, false, item) ;
  
        } 
        var done = data.done ;
        for (var i = 0 ; i < done.length ; i++) {
          var item = done[i] ;
          display_item(t, true, item) ;
        } 
      }
    ) ;
  
  }


And data for refreshing is sent in
((<JSON|URL:http://www.crockford.com/JSON/index.html>)) format.  This
is very simple with
((<ruby-json|URL:http://sourceforge.net/projects/json>)) library.

  def json_data
    not_done = Todo.find_not_done
    done = Todo.find_done
    data = { 
      :not_done => not_done.collect do |r|
        { 
          :description=>r.description,
          :item_done=>r.done,
          :id=> r.id 
        }
      end,
      :done => done.collect do |r|
        { 
          :description=>r.description,
          :item_done=>r.done,
          :id=> r.id 
        }
      end
    }
    json_response = { :ret => 200, :body => data }
    headers["Content-Type"] = 'text/javascript'
    render_text('json_response = ' + json_response.to_json)
  end

