require 'rubygems'
require 'json/objects'
require 'amrita2/util'
require 'amrita2/core'

module Amrita2
  module JS
    RUNTIME_SRC = SanitizedString[<<END]

function toString(obj) { 
      message = ""
      for (i in obj) { 
        message += i + ":" + obj[i] + "\t"
      }
      return message ;
}

function amrita_appendText(parent, text) { 
  debug('amrita_appendText');
  return parent.appendChild(document.createTextNode(text)) ;
}

function amrita_appendElement(parent, name, attr, body) { 

  debug('amrita_appendElement');
/*
  debug(parent.to_string) ;
  debug(name) ;
  debug(attr.to_string) ;
*/
  var e = document.createElement(name) ;
  for (var k in attr) { 
    e.setAttribute(k, attr[k]) ;
  } ;
  switch(typeof (body)) { 
  case "string":
    amrita_appendText(e, body)
    break ;
  case "function":
    amrita_appendText(e, body()) ;
    break ;
  case "undefined":
    break ;
  default:
    amrita_appendText(e, body) ;
    break ;
  }
  return parent.appendChild(e) ;
}

function amrita_moveChildren(from ,to) { 
  debug('amrita_moveChildren') ;
  var c = from.childNodes ;
  for(; c.length > 0;) { 
    to.appendChild(c[0]) ;
  }
}

function amrita_data_to_func(data) { 
  function sub(tmpl, key, data) { 
    debug('amrita_data_to_func::sub ' + typeof data);
/*
    debug(key) ;
    debug(data) ;
*/
    switch(typeof data) { 
    case 'function' :
      eval('tmpl.' + key + '(data)') ;
      break ;
    case 'object':
      if (typeof data[0] != 'undefined') { 
        for(var i = 0 ; i < data.length ; i++) { 
          sub(tmpl, key, data[i]) ;
        }
      } else { 
        var f = function(t) { 
          for(var d in data) { 
            var dd = data[d] ;
            sub(tmpl, d, dd) ;
          }
        } ;
        eval('tmpl.' + key + '(f)') ;
      }
      break ;
    default:
      eval('tmpl.' + key + '(data)') ;
      break ;
    }
  }
  // debug(typeof data) ;
  var ret = data ;
  switch(typeof data) { 
  case 'function' :
    break ;
  case 'object':
    ret = function(t) { 
      for(var d in data) { 
        var dd = data[d] ;
        sub(t, d, dd) ;
      }
    } ;
    break ;
  default:
    debug('unknown data' + typeof data) ;
    break ;
  }
  return ret ;
}


END

    include TextSupport

    class CodeGenerator
      include TextSupport
      def initialize
        init_results
        @current_indent = 0
        @varcnt = 0
      end

      def new_var
        @varcnt += 1
        "v#{@varcnt}"
      end

      def code(src)
        return if src == nil or src == ""
        src = src.join(NL) if src.kind_of?(Array)
        unindent_auto(src).each do |l|
          @results << [@current_indent, l.chomp]
        end
      end

      def results
        @results.collect do |level, line|
          "  " * level + line
        end
      end

      def if_(cond)
        code("if (#{cond}) {")
        level_up do
          yield
        end
        code("}")
      end

      def else_
        level_down do
          code("} else {")
        end
        yield
      end

      def level_up
        @current_indent += 1
        yield
      ensure
        @current_indent -= 1
      end

      def level_down
        @current_indent -= 1
        yield
      ensure
        @current_indent += 1
      end

      def define_noname_function(*params, &block)
        define_function(nil, *params, &block)
      end

      def define_function(name, *params)
        if params.size == 0
          code("function #{name}() {")
        else
          code("function #{name}(#{params.join(',')}) {")
        end
        level_up do
          yield
        end
        code("}")
        self
      end

      def call(*args)
        a = ''
        a = args.join(',') if args.size > 0
        code "(#{a}) ;"
      end

      def element(e)
        v = new_var
        define_noname_function do 
          code  %[var #{v} = document.createElement("#{e.name}") ; ]
          e.attributes.each do |k,val|
            code %[#{v}.setAttribute("#{k}", "#{val}") ; ]
          end
        
          if block_given?
            code %[#{v}.innerHTML = "#{yield}" ; ]
          else
            e.each do |c|
              #code %[debug("#{c.class}") ;]
              case c
              when REXML::Text
                code %[#{v}.appendChild(document.createTextNode("#{c}")) ;]
              when REXML::Element
                vv = new_var
                code "var #{vv} = "
                element(c)
                code "#{v}.appendChild(#{vv}) ;"
              else
                raise "not supported #{ c.class }"
              end
            end
          end
          code %[return #{v} ; ]
        end.call
      end

      private

      def init_results
        @results = []
      end

    end

    module SpecMixin
      def compile_js_main(e, cg)
        if e.amrita_spec 
          e.amrita_spec.comile_js_main_dynamic(e, cg)
        else
          compile_js_main_static(e, cg)
        end
      end

      def comile_js_main_dynamic(e, cg)
        cg.code %[amrita_moveChildren(this.stream.#{sym}, parent)]
      end

      def compile_js_main_static(e, cg)
        if e.size > 0
          cg.code %[parent = amrita_appendElement(parent, #{e.name.to_json}, #{e.attributes.to_json})]
          e.each_child do |c|
            compile_js_main_node(c, cg)
          end
        else
          cg.code %[amrita_appendElement(parent, #{e.name.to_json}, #{a}) ; ]
        end
      end

      def compile_js_main_node(n, cg)
        case n
        when REXML::Text
          cg.code %[amrita_appendText(parent, #{n.to_s.to_json}) ;]
        when REXML::Element
          if n.amrita_spec
            n.amrita_spec.compile_js_main(n, cg)
          else
            compile_js_main_static(n, cg)
          end
        end
      end

      def compile_js_sub(e, cg)
        if e.amrita_spec
          e.amrita_spec.compile_js_sub_dyn(e, cg)
        end
      end

      def compile_js_sub_dyn(e, cg)
        if children.size > 0
          compile_for_not_leaf(e, cg)
        else
          compile_for_leaf(e, cg)
        end
      end

      def compile_for_not_leaf(e, cg)
        cg.code %[debug('compile_for_not_leaf') ;]
        cg.code %[debug(#{sym.to_json}) ;]
        cg.code %[this.#{module_name}  = ]
        cg.level_up do
          cg.define_noname_function('parent', 'func') do
            e.elements.each do |c|
              compile_js_sub(c, cg)
            end
            cg.code %[return this ; ]
          end
        end
        cg.code %[this.stream.#{sym} = document.createElement("dummy") ; ]
        cg.code %[this.#{sym} = ]
        cg.level_up do
          cg.define_function(nil, 'attr', 'func') do
            cg.code <<-END
            debug(#{sym.to_json}) ;
            if (typeof func == "undefined") { 
               func = attr ;
            }
            if (typeof func != "undefined") { 
              var f = amrita_data_to_func(func) ;
              f(this.#{module_name}()) ;
            }
            var parent = amrita_appendElement(this.stream.#{sym}, #{e.name.to_json}, #{e.attributes.to_json}) ;
            // debug(typeof attr) ;
            END
            e.each_child do |c|
              compile_js_main_node(c, cg)
            end
            cg.code %[return this ;]
          end
        end        
        cg.code %[;]
      end

      def compile_for_leaf(e, cg)
        cg.code %[debug('compile_for_leaf') ;]
        cg.code %[debug(#{sym.to_json}) ;]
        cg.code %[this.stream.#{sym} = document.createElement("dummy") ; ]
        cg.code %[this.#{sym} = ]
        cg.level_up do
          cg.define_function(nil, 'val', 'attr') do
            cg.code <<-END
            debug(#{sym.to_json}) ;
            /*
            debug(val) ;
            debug(attr) ;
            */
            var ee = amrita_appendElement(this.stream.#{sym}, #{e.name.to_json}, #{e.attributes.to_json}, val) ;
            debug(typeof attr) ;
            if (typeof attr != "undefined") { 
              for(var k in attr) { 
                // debug(k) ;
                // debug(attr[k]) ;
                ee.setAttribute(k, attr[k]) ;
              }
            }
            return this ;
            END
          end
        end
        cg.code %[;]
      end
    end

    module RootSpecMixin
      include SpecMixin
      def create_js_template_src(name, root, cg)
        cg.define_function(name) do 
          cg.code %[this.expand  = ]
          cg.level_up do
            cg.define_noname_function('parent', 'func') do
              cg.code %[var f = amrita_data_to_func(func) ;]
              cg.code %[var ret = parent ;]
              cg.code %[f(this.#{module_name}()) ;]
              compile_js_main(root, cg)
              cg.code %[return ret ; ]
            end
          end
          cg.code ";"
          cg.code %[this.#{module_name} = ]
          cg.level_up do
            cg.define_noname_function do
              cg.code %[this.stream = {} ; ]
              root.elements.each do |c|
                compile_js_sub(c, cg)
              end
              cg.code %[return this ;]
            end
          end
          cg.code ";"
          parts.each do |modname, method_name, spec|
            create_parts_src(cg, modname, method_name, spec)
          end
        end
      end

      # This is not perfect !
      # This workds for only limitted case
      def create_parts_src(cg, modname, method_name, spec)
        has_children = (spec.children.size > 0)
        if spec.children.size > 0
          cg.code %[this.#{method_name} = ]
          cg.define_noname_function('d', 'a') do 
            cg.level_up do
              cg.code <<-END
              var x = new this.MainContext() ;
              var y = x.#{spec.module_name}()  ;
              y.#{spec.sym}(d, a) ;
              return y.stream.#{spec.sym}.firstChild ;
              END
            end
          end
          cg.code ";"
        else
          cg.code %[this.#{method_name} = ]
          cg.define_noname_function('d', 'a') do 
            cg.level_up do
              cg.code <<-END
              var x = new this.MainContext() ;
              x.#{spec.sym}(d, a) ;
              return x.stream.#{spec.sym}.firstChild ;
              END
            end
          end
          cg.code ";"
        end
      end
    end

    class Template < Amrita2::Template::Template
      def js_template(name, cg)
        setup unless @setuped
        get_spec.create_js_template_src(name, @root, cg)
      end
    end

    class TemplateText < Template
      def initialize(text, &block)
        super(&block)
        @text = text
      end

      def get_document
        REXML::Document.new(@text)
      end
    end

    class TemplateFile < Template
      def initialize(fname, &block)
        super(&block)
        @fname = fname
      end
      
      def get_document
        REXML::Document.new(File.new(@fname))
      end
    end
  end

  class Core::DynamicElementSpec
    include JS::SpecMixin
  end

  class Core::RootSpec
    include JS::RootSpecMixin
  end
end


