require "amrita2/js"

class TodoController < ApplicationController
  model :todo
  scaffold :todo

  # done and not_done were added to the original controller
  attr_reader :done, :not_done

  def list
    @not_done = Todo.find_not_done
    @done = Todo.find_done
  end

  def add_item 
    #render_text @params.inspect
    item = Todo.new  # Create a new instance of Todo, so create a new item
    item.attributes = @params["new_item"]  # The fields of item should be set to what's in the "new_item" hash
    if item.save  # Try to save our item into the database
      redirect_to(:action => "json_list")  # Return to the list page if it suceeds
    else
      render_text "Couldn't add new item"  # Print an error message otherwise
    end
  end

  def destroy
    item = Todo.find(@params["id"])
    begin
      item.destroy
      redirect_to(:action => "json_list")
    rescue ActiveRecord::ActiveRecordError
      render_text "Couldn't destroy item" 
    end
  end

  def toggle_check
    item = Todo.find(@params["id"])

    # change the done flag.  ActiveRecord
    # will automatically do the boolean/int
    # conversion.  
    item.done = !item.done?

    if item.save
      redirect_to(:action => "json_list")
    else
      render_text "Couldn't update item" 
    end
  end

  def json_data
    not_done = Todo.find_not_done
    done = Todo.find_done
    data = { 
      :not_done => not_done.collect do |r|
        { 
          :description=>r.description,
          :item_done=>r.done,
          :id=> r.id 
        }
      end,
      :done => done.collect do |r|
        { 
          :description=>r.description,
          :item_done=>r.done,
          :id=> r.id 
        }
      end
    }
    json_response = { :ret => 200, :body => data }
    headers["Content-Type"] = 'text/javascript'
    render_text('json_response = ' + json_response.to_json)
  end

  def json_list
    list
  end
end
