module TodoHelper
  # Presentation Object for Todo Record
  class TodoPO
    def initialize(r)
      @r = r
    end

    # passive PO method: 
    # The result of this method will be inserted in HTML output.
    def description
      @r.description
    end

    # active PO method:
    # m is a template Module with method for dynamic element.
    # If a method has a parameter for it, one of methods of that module
    # should be called by PO object.
    def item_done(m)
      m.item_done(nil, :onclick => %[document.location.href="/todo/toggle_check/#{@r.id}"])
    end

    def edit_link(m)
      m.edit_link("Edit", :href=>"/todo/edit/#{@r.id}")
    end

    def destroy_link(m)
      m.destroy_link("Destroy", :href=>"/todo/destroy/#{@r.id}",
                     :onclick=>%[return confirm("Are you sure you want to delete this entry: #{description}")])
    end
  end

  # Presentation Object for 'list' method
  # The result of this method will be used by Amrita2 for tempalte expansion
  def list_po
    {
      :done => controller.done.collect do |r|
        TodoPO.new(r)
      end,
      :not_done => controller.not_done.collect do |r|
        TodoPO.new(r)
      end
    }
  end

  def json_list_script
    #path = File::expand_path("~/amrita2/sample/rails/todoajax/views/todo/json_list.a2html")
    #tmpl =Amrita2::JS::TemplateFile.new(path)

    tmpl = Amrita2::JS::TemplateText.new <<-END
    <div>
    Not done:
    <div id='not_done'>
       <input id="item_done" name="item[done]" type="checkbox" value="1" />
       <span> </span>
       <span id='description' />
       <span> </span>
       <a id='edit_link' href="/todo/edit/">Edit</a> 
       <span> </span>
       <a id='destroy_link' href="/todo/edit/">Destroy</a> 
    </div>
    Done:
    <div id='done'>
       <input checked="checked" id="item_done" name="item[done]" type="checkbox" value="0" />
       <span> </span>
       <span id='description' />
       <span> </span>
       <a id='edit_link' href="/todo/edit/">Edit</a> 
       <span> </span>
       <a id='destroy_link' href="/todo/edit/">Destroy</a> 
    </div>
    <span id='now' />
    </div>
    END
    # compiling the template to JavaScript functions
    js = Amrita2::JS::CodeGenerator.new
    tmpl.js_template('tmpl', js)
    Amrita2::JS::RUNTIME_SRC + js.results.join("\n") 
  end

  def json_list_po
    {
      :done => controller.done.collect do |r|
        TodoPO.new(r)
      end,
      :not_done => controller.not_done.collect do |r|
        TodoPO.new(r)
      end,
      :output=>true,
      :script=>Amrita2::SanitizedString[json_list_script]
    }
  end
end

