
require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2'
require 'rexml/document'

# ruby -I~/amrita2/lib ~/amrita2/test/test_ja.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

$KCODE='e'
module Amrita2JapaneseTest
  include Amrita2
  include Amrita2::Core

  class TestJapanese < Test::Unit::TestCase
    def test_ja
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa, :encoding=>"euc-jp") do
          dynamic_element(:bbb, :replace_attr=>true)
        end
      end
      doc = REXML::Document.new  "<?xml version='1.0' encoding='EUC-JP'?><x id='aaa'><y id='bbb' /></x>"
      t = spec.create_template(doc)

      out = ""
      t.expand_template(out) do |m|
        m.aaa do |ma|
          ma.bbb('a')
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb'>a</y></x>", out)

      t.expand_template(out="") do |m|
        m.aaa do |ma|
          s = 'ܸ'
          ma.bbb(s)
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb'>ܸ</y></x>", out)
      t.expand_template(out="") do |m|
        m.aaa do |ma|
          ma.bbb('ܸ', :atr=>"ܸ°")
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb' atr='ܸ°'>ܸ</y></x>", out)
    end

    def test_sjis
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa, :encoding=>"shift_jis") do
          dynamic_element(:bbb, :replace_attr=>true)
        end
      end
      sjis_txt = Iconv::iconv("shift_jis", "euc-jp", "<?xml version='1.0' encoding='SHIFT_JIS'?><x id='aaa'><y id='bbb' /></x>")[0]
      doc = REXML::Document.new  sjis_txt
      t = spec.create_template(doc)

      out = ""
      t.expand_template(out) do |m|
        m.aaa do |ma|
          ma.bbb('a')
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb'>a</y></x>", out)

      t.expand_template(out="") do |m|
        m.aaa do |ma|
          s = 'ܸ'
          s = Iconv::iconv("shift_jis", "euc-jp", s)[0]
          ma.bbb(s)
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb'>ܸ</y></x>", out)
      t.expand_template(out="") do |m|
        m.aaa do |ma|
          s = 'ܸ'
          s = Iconv::iconv("shift_jis", "euc-jp", s)[0]
          a = 'ܸ°'
          a = Iconv::iconv("shift_jis", "euc-jp", a)[0]
          ma.bbb(s, :atr=>a)
        end
      end
      out = Iconv::iconv("euc-jp", "utf-8", out)[0]
      assert_equal("<x id='aaa'><y id='bbb' atr='ܸ°'>ܸ</y></x>", out)
    end
  end
end

if __FILE__ == $0
  include Amrita2JapaneseTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestJapanese.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
