
require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/specgenerator'

# ruby -I~/amrita2/lib ~/amrita2/test/test_specgenerator.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

module Amrita2SpecGenTest
  include Amrita2

  class TestGenerator < Test::Unit::TestCase
    def compare_element(a, b)
      assert_equal(a.name, b.name)
      assert_equal(a.elements.size, b.elements.size)
      ae = a.elements
      if ae.size > 0
        be = b.elements
        ae.each_with_index do |e, i|
          compare_element(e, be[i+1])
        end
      else
        assert_equal(a.text, b.text)
      end
    end

    def compare_result(a, b)
      adoc = REXML::Document.new(a)
      bdoc = REXML::Document.new(b)
      compare_element(adoc.root, bdoc.root)
    rescue
      puts
      puts '------a------------'
      puts a
      puts '------b------------'
      puts b
      puts '-------------------'
      raise
    end

    def do_test(tmpl, opt={})
      doc = REXML::Document.new(tmpl)
      g = SpecGenerator.new(doc, opt)
      src = g.generate_all
      libpath = File::join(File::dirname(__FILE__), "../lib")
      IO.popen("ruby -I #{libpath}", "r+") do |io|
        io.print src
        io.close_write
        result = io.read
        compare_result(tmpl, result)
      end
    rescue
      puts
      puts '-------------------'
      puts src
      puts '-------------------'
      raise
    end

    def test_leaf
      doc = REXML::Document.new  "<x id='aaa'>111</x>"
      g = SpecGenerator.new(doc)
      g.convert(out="")
      assert_equal('dynamic_element(:aaa) # 111 ', out.chomp)
    end

    def test_not_leaf
      doc = REXML::Document.new  "<x id='aaa'><y id='bbb'>111</y></x>"
      g = SpecGenerator.new(doc)
      g.convert(out="")
      assert_equal("dynamic_element(:aaa) do\n  dynamic_element(:bbb) # 111 \nend\n", out)
    end

    def test_not_leaf2
      doc = REXML::Document.new  "<x id='aaa'><y id='bbb'><z id='ccc'>111</z></y></x>"
      g = SpecGenerator.new(doc)
      g.convert(out="")
      assert_equal("dynamic_element(:aaa) do\n  dynamic_element(:bbb) do\n    dynamic_element(:ccc) # 111 \n  end\nend\n", out)
    end

    def test_multi_key
      #doc = REXML::Document.new  "<x id='aaa'><y class='bbb'><z xx='ccc'>111</z></y></x>"
      doc = REXML::Document.new  "<x id='aaa'><y class='bbb'><z xx='ccc'>111</z></y></x>"
      g = SpecGenerator.new(doc)
      g.add_key(:class)
      g.add_key(:id)
      g.convert(out="")
      assert_equal(%[dynamic_element(:aaa) do\n  dynamic_element(:bbb, :matcher => Amrita2::Core::AttrMatcher.new(:class, "bbb")) # <z xx='ccc'>111</z> \nend\n], out)

      g = SpecGenerator.new(doc)
      g.add_key(:class)
      g.add_key(:id)
      g.add_key(:xx)
      g.convert(out="")
      assert_equal("dynamic_element(:aaa) do\n  dynamic_element(:bbb, :matcher => Amrita2::Core::AttrMatcher.new(:class, \"bbb\")) do\n    dynamic_element(:ccc, :matcher => Amrita2::Core::AttrMatcher.new(:xx, \"ccc\")) # 111 \n  end\nend\n", out)
    end

    def test_simple
      do_test "<x id='aaa'>111</x>"      
    end

    def test_nest
      do_test "<x id='aaa'><y id='bbb'>111</y></x>"
    end

    def test_nest2
      do_test "<x id='aaa'><y id='bbb'>111</y><y id='bbb'>222</y></x>"
    end

    def test_nest3
      do_test "<outer><x id='aaa'><y id='bbb'>111</y></x><x id='aaa'><y id='bbb'>111</y></x></outer>"
    end

    def test_class
      do_test "<outer><x class='aaa'><y id='bbb'>111</y></x><x id='aaa'><y id='bbb'>111</y></x></outer>", :key_attr_name=>:class
      do_test "<outer><x class='aaa'><y class='bbb'>111</y></x><x class='aaa'><y class='bbb'>222</y></x></outer>", :key_attr_name=>:class
    end

    def test_hello
      do_test <<-END
      <html>
      <body>
      <h1 id='title'>title will be inserted here</h1>
      <p id='body'>body text will be inserted here</p>
      </body>
      </html>
      END
    end

    def test_table
      do_test <<-END
      <html>
      <body id='body'>
      <table id='table' border='1'>
        <tr><th>name</th><th>author</th></tr>
        <tr id='d'>
          <td id='a'>Ruby</td><td id='b'>matz</td>
        </tr><tr id='d'>
          <td id='a'>perl</td><td id='b'>Larry Wall</td>
        </tr><tr id='d'>
          <td id='a'>python</td><td id='b'>Guido van Rossum</td>
        </tr>
      </table>
      </body></html>
      END
    end

    def test_notag_element
      do_test <<-END
      <html>
      <body>
      <h1 id='title'>title will be inserted here</h1>
      <p id='body'>body text will be inserted <a href='xxx'>here</a></p>
      </body>
      </html>
      END
    end

  end
end

if __FILE__ == $0
  include Amrita2SpecGenTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestGenerator.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
